/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.exception.ComponentComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.ComponentErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component.
 *
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsComponentComponentClient
		extends
			PetalsAbstractServiceClient {

	/**
	 * Component states
	 */
	public final static class State {

		public final static String STARTED = "Started";

		public final static String STOPPED = "Stopped";

		public final static String SHUTDOWN = "Shutdown";

	}

	/**
	 * Action : shutdown a component
	 */
	protected static final String SHUTDOWN = "shutDown";

	/**
	 * Action : start a component
	 */
	protected static final String START = "start";

	/**
	 * Action : stop a component
	 */
	protected static final String STOP = "stop";

	/**
	 * Attribut : current state
	 */
	protected static final String CURRENT_STATE = "CurrentState";


	/**
	 * Creates a new instance of {@link PetalsComponentComponentClient}
	 *
	 * @param petalsDomain
	 * @param componentName
	 * @param mBeanServerConnection
	 * @throws ComponentComponentDoesNotExistException
	 * @throws ComponentErrorException
	 */
	public PetalsComponentComponentClient(String petalsDomain,
			String componentName, MBeanServerConnection mBeanServerConnection)
			throws ComponentComponentDoesNotExistException,
			ComponentErrorException {

		super(petalsDomain, mBeanServerConnection);

		try {
			Set<ObjectName> objNames;

			String pattern = "*:type=engine,name=" + componentName;
			ObjectName objName = new ObjectName(pattern);
			objNames = this.mBeanServerConnection.queryNames(objName, null);
			if (objNames != null && objNames.size() == 1) {
				this.mbeanName = objNames.iterator().next();
			} else {
				pattern = "*:type=binding,name=" + componentName;
				objName = new ObjectName(pattern);
				objNames = this.mBeanServerConnection.queryNames(objName, null);
				if (objNames != null && objNames.size() == 1) {
					this.mbeanName = (ObjectName) objNames.iterator().next();
				} else {
					throw new ComponentComponentDoesNotExistException(
							componentName);
				}
			}

		} catch (MalformedObjectNameException e) {
			throw new ComponentErrorException(e);
		} catch (IOException e) {
			throw new ComponentErrorException(e);
		}
	}

	/**
	 * Start the component associated to this JMX component.
     *
	 * @throws PerformActionErrorException
	 */
	public void start() throws PerformActionErrorException {

		Object[] objects = new Object[0];
		String[] strings = new String[0];
		this.performAction(START, objects, strings);

	}

	/**
	 * Stop the component associated to this JMX component.
	 *
	 * @throws PerformActionErrorException
	 */
	public void stop()
			throws PerformActionErrorException {

		Object[] objects = new Object[0];
		String[] strings = new String[0];
		this.performAction(STOP, objects, strings);

	}

	/**
	 * Shutdown the component associated to this JMX component.
	 *
	 * @throws PerformActionErrorException
	 */
	public void shutdown() throws PerformActionErrorException {

		Object[] objects = new Object[0];
		String[] strings = new String[0];
		this.performAction(SHUTDOWN, objects, strings);

	}

	/**
	 * Get the current state of the JBI component associated to this JMX
	 * component.
	 *
	 * @return the current state of the JBI component.
	 * @throws AttributeErrorException
	 */
	public String getState() throws AttributeErrorException {

		return (String) this.getAttribute(CURRENT_STATE);

	}

}
